"""File to store constants."""
ALL_OPTION = "ALL"
NONE_OPTION = "NONE"
STATIC_URL_PREFIX = "vizro"
MODULE_PAGE_404 = "not_found_404"
EMPTY_SPACE_CONST = -1
ON_PAGE_LOAD_ACTION_PREFIX = "on_page_load_action"
FILTER_ACTION_PREFIX = "filter_action"
PARAMETER_ACTION_PREFIX = "parameter_action"
ACCORDION_DEFAULT_TITLE = "SELECT PAGE"
