:root {
  --primary-dark-100: #333640;
  --primary-dark-200: #2f323c;
  --primary-dark-300: #2b2e39;
  --primary-dark-400: #272a35;
  --primary-dark-500: #232632;
  --primary-dark-600: #1f222e;
  --primary-dark-700: #1b1e2a;
  --primary-dark-800: #181b26;
  --primary-dark-900: #141721;
  --surfaces-dark-mode-bg-01: rgba(35, 38, 50, 1);
  --surfaces-dark-mode-bg-02: rgba(27, 30, 42, 1);
  --surfaces-dark-mode-bg-03: rgba(20, 23, 33, 1);
  --surfaces-dark-mode-bg-card: rgba(35, 38, 50, 1);
  --border-dark-mode-disabled: rgba(255, 255, 255, 0.3);
  --field-dark-mode-disabled: rgba(240, 241, 242, 1);
  --fill-dark-mode-disabled: rgba(255, 255, 255, 0.3);
  --text-dark-mode-disabled: rgba(255, 255, 255, 1);
  --border-dark-mode-enabled: rgba(255, 255, 255, 0.6);
  --field-dark-mode-enabled: rgba(43, 46, 57, 1);
  --status-dark-mode-error: rgba(245, 101, 101, 1);
  --fill-dark-mode-high-emphasis: rgba(255, 255, 255, 0.88);
  --border-dark-mode-hover: rgba(255, 255, 255, 0.88);
  --field-dark-mode-hover: rgba(55, 58, 68, 1);
  --fill-dark-mode-hover-selected: rgba(255, 255, 255, 1);
  --status-dark-mode-information: rgba(0, 180, 255, 1);
  --fill-dark-mode-medium-emphasis: rgba(255, 255, 255, 0.6);
  --text-dark-mode-placeholder: rgba(255, 255, 255, 0.38);
  --text-dark-mode-primary: rgba(255, 255, 255, 0.88);
  --text-dark-mode-secondary: rgba(255, 255, 255, 0.6);
  --border-dark-mode-selected: #fff;
  --fill-dark-mode-subtle: rgba(255, 255, 255, 0.1);
  --border-dark-mode-subtle-alpha-01: rgba(255, 255, 255, 0.1);
  --status-dark-mode-success: rgba(64, 216, 110, 1);
  --tags-dark-mode-text-color: rgba(255, 255, 255, 0.6);
  --status-dark-mode-warning: rgba(255, 193, 7, 1);
  --primary-light-100: #fafafb;
  --primary-light-200: #f5f6f6;
  --primary-light-300: #f2f3f4;
  --primary-light-400: #ebedee;
  --primary-light-50: #ffffff;
  --primary-light-500: #e6e8ea;
  --primary-light-600: #e2e4e6;
  --primary-light-700: #dddfe1;
  --primary-light-800: #d8dadd;
  --primary-light-900: #d3d6d9;
  --surfaces-light-mode-bg-01: rgba(255, 255, 255, 1);
  --surfaces-light-mode-bg-02: rgba(245, 246, 246, 1);
  --surfaces-light-mode-bg-03: rgba(230, 232, 234, 1);
  --surfaces-light-mode-bg-card: rgba(245, 246, 246, 1);
  --border-light-mode-disabled: rgba(20, 23, 33, 0.3);
  --field-light-mode-disabled: rgba(240, 241, 242, 1);
  --fill-light-mode-disabled: rgba(20, 23, 33, 0.3);
  --text-light-mode-disabled: rgba(20, 23, 33, 0.3);
  --border-light-mode-enabled: rgba(20, 23, 33, 0.6);
  --field-light-mode-enabled: rgba(250, 250, 251, 1);
  --status-light-mode-error: rgba(240, 59, 58, 1);
  --fill-light-mode-high-emphasis: rgba(20, 23, 33, 0.88);
  --border-light-mode-hover: rgba(20, 23, 33, 0.88);
  --field-light-mode-hover: rgba(255, 255, 255, 1);
  --fill-light-mode-hover-selected: rgba(20, 23, 33, 1);
  --status-light-mode-information: rgba(0, 158, 255, 1);
  --fill-light-mode-medium-emphasis: rgba(20, 23, 33, 0.6);
  --text-light-mode-placeholder: rgba(20, 23, 33, 0.38);
  --text-light-mode-primary: rgba(20, 23, 33, 0.88);
  --text-light-mode-secondary: rgba(20, 23, 33, 0.6);
  --border-light-mode-selected: #141721;
  --fill-light-mode-subtle: rgba(0, 0, 0, 0.1);
  --border-light-mode-subtle-alpha-01: rgba(20, 23, 33, 0.1);
  --status-light-mode-success: rgba(38, 191, 86, 1);
  --tags-light-mode-text-color: rgba(255, 255, 255, 0.6);
  --status-light-mode-warning: rgba(241, 124, 2, 1);
  --overlays-on-light-ui-high-contrast: rgba(239, 240, 248, 0.88);
  --overlays-on-light-ui-medium-contrast: rgba(239, 240, 248, 0.6);
  --state-overlays-dark-mode-active: rgba(255, 255, 255, 0.08);
  --state-overlays-dark-mode-enable: rgba(255, 255, 255, 0);
  --state-overlays-dark-mode-hover: rgba(255, 255, 255, 0.12);
  --state-overlays-dark-mode-selected: rgba(255, 255, 255, 0.1);
  --state-overlays-dark-mode-selected-hover: rgba(255, 255, 255, 0.16);
  --state-overlays-light-mode-active: rgba(20, 23, 33, 0.12);
  --state-overlays-light-mode-enable: rgba(20, 23, 33, 0);
  --state-overlays-light-mode-hover: rgba(20, 23, 33, 0.08);
  --state-overlays-light-mode-selected: rgba(20, 23, 33, 0.08);
  --state-overlays-light-mode-selected-hover: rgba(20, 23, 33, 0.24);
}
