textarea {
  background-color: var(--field-enabled);
  padding: var(--spacing-02);
}

textarea:focus {
  box-shadow: 0 0 0 2px var(--focus-focus) inset;
  outline: none;
}

hr {
  border-bottom: 1px solid var(--border-subtle-alpha-01);
  width: 100%;
}

blockquote {
  border-left: var(--spacing-01) var(--text-secondary) solid;
  padding-left: var(--spacing-01);
}
