@font-face {
  font-family: "Inter";
  src: url("./fonts/inter-variable-font.ttf") format("truetype");
}

@font-face {
  font-family: "RobotoMono";
  src: url("./fonts/roboto-mono-variable-font.ttf") format("truetype");
}

* {
  border: 0;
  box-sizing: border-box;
  font-family: Inter, Sans-Serif, Arial, serif;
  letter-spacing: 0;
  margin: 0;
  text-decoration: none;
}
