/* Outer layout for dash dropdown */
#page-container .dash-dropdown {
  background-color: var(--field-enabled);
  border: none;
  border-radius: 0;
  box-shadow: var(--box-shadow-elevation-0);
  font-size: var(--text-size-02);
}

/* Dropdown menu when clicking on expand arrow */
#page-container .Select-menu-outer {
  background-color: var(--field-enabled);
  border: none;
  border-radius: 0;
  box-shadow: var(--box-shadow-elevation-0);
  font-size: var(--text-size-02);
  line-height: var(--text-size-05);
}

/* Dropdown menu options */
#page-container .VirtualizedSelectOption {
  color: var(--text-primary);
  font-weight: var(--text-weight-regular);
}

/* Dropdown menu hover effect */
#page-container .VirtualizedSelectFocusedOption {
  background-color: var(--state-overlays-hover);
}

/* Input box for existing values and user input */
#page-container .Select-control {
  background-color: inherit;
  border: inherit;
  border-radius: inherit;
  height: 32px;
}

#page-container .Select.is-focused > .Select-control {
  background-color: var(--field-enabled);
}

/* User input */
#page-container .dash-dropdown .Select-input {
  display: block;
  height: var(--tag-height);
  margin-left: unset;
}

#page-container.Select-input > input {
  height: 100%;
  padding: 0;
}

/* Border on focus */
#page-container .is-focused:not(.is-open) > .Select-control {
  box-shadow: 0 0 0 2px var(--focus-focus) inset;
}

/* Single-select dropdown only ------------------- */
#page-container .Select--single .Select-value {
  padding-left: 8px;
}

.has-value.Select--single > .Select-control .Select-value .Select-value-label,
.has-value.is-pseudo-focused.Select--single
  > .Select-control
  .Select-value
  .Select-value-label {
  color: var(
    --text-primary
  ) !important; /* Required so text color don't change caused by adding table */
}

/* Tags ---------------------------*/
#page-container .Select--multi .Select-value {
  background-color: var(--tags-bg-color);
  border: 0;
  border-radius: 0;
  box-shadow: var(--box-shadow-elevation-0);
  color: var(--tags-text-color);
  font-size: var(--text-size-02);
  height: var(--tag-height);
  letter-spacing: var(--letter-spacing-body-ui-02);
  margin: 0;
  padding: 0 var(--spacing-01);
  vertical-align: baseline;
}

/* Tag: Label */
#page-container .Select--multi .Select-value-label {
  color: var(--text-secondary);
  line-height: var(--text-size-05);
  padding: 0 4px;
}

/* Tag: Cross */
#page-container .Select--multi .Select-value-icon {
  color: var(--text-secondary);
  background: inherit;
  border: none;
  line-height: inherit;
  margin: 0;
  padding: 0;
}

/* Tag: Cross on hover */
#page-container .Select--multi .Select-value-icon:hover {
  background: inherit;
  color: var(--text-secondary);
}

/* Tag: Arrow */
.Select-arrow-zone {
  padding-right: 8px;
  width: 16px;
}

/* Page Selector Dropdown */
#page-container
  .dash-dropdown
  .Select-menu-outer
  .VirtualizedSelectSelectedOption
  > a {
  pointer-events: none;
}

#page-container .dash-dropdown .Select-menu-outer .VirtualizedSelectOption > a {
  height: 100%;
  width: 100%;
}

#nav-panel a {
  font-weight: var(--text-weight-regular);
  line-height: var(--text-size-07);
}

#page-container .Select--multi .Select-clear-zone {
  width: 16px;
  color: var(--text-secondary);
}

/** spacing for tag elements only when elements are present inside
wrapper **/
.Select--multi.has-value .Select-multi-value-wrapper {
  display: flex;
  flex-wrap: wrap;
  gap: 4px;
  padding: 4px 8px;
}

.Select-input > input {
  padding: 0 !important; /* Required so tags don't jump caused by adding table */
}
