#right-side
  .dash-table-container
  .dash-spreadsheet-container
  .dash-spreadsheet-inner
  th {
  background-color: var(--main-container-bg-color);
  color: var(--text-primary);
}

#right-side
  .dash-table-container
  .dash-spreadsheet-container
  .dash-spreadsheet-inner
  td {
  background-color: var(--main-container-bg-color);
  color: var(--text-primary);
}

#right-side
  .dash-table-container
  .dash-spreadsheet-container
  .dash-spreadsheet-inner
  table {
  --hover: var(--main-container-bg-color);
}

#right-side
  .dash-table-container
  .dash-spreadsheet-container
  .dash-spreadsheet-inner
  td
  .dash-cell-value.unfocused {
  font-size: 14px;
}

#right-side
  .dash-table-container
  .dash-spreadsheet-container
  .dash-spreadsheet-inner
  .column-header-name {
  font-size: 14px;
}

.table-container {
  height: 100%;
  overflow: auto;
  padding: 12px;
  padding-left: 32px;
  width: 100%;
}
